"""
Encapsulates the settings for the conversation history of an agent.
"""
from typing import Optional
from pydantic import BaseModel

class AgentConversationHistorySettings(BaseModel):
    """Agent Conversation History Settings."""
    enabled: Optional[bool] = False
    max_history: Optional[int] = 10



"""FoundationaLLM orchestration models module"""
from .analysis_result import AnalysisResult
from .content_artifact import ContentArtifact

from .attachment_detail import AttachmentDetail
from .file_history_item import FileHistoryItem
from .message_content_item_types import MessageContentItemTypes
from .message_content_item_base import MessageContentItemBase

from .openai_file_path_message_content_item import OpenAIFilePathMessageContentItem
from .openai_image_file_message_content_item import OpenAIImageFileMessageContentItem
from .openai_text_message_content_item import OpenAITextMessageContentItem

from .completion_request_object_keys import CompletionRequestObjectKeys
from .completion_request_base import CompletionRequestBase
from .completion_response import CompletionResponse



from .openai_assistants_request import OpenAIAssistantsAPIRequest
from .openai_assistants_response import OpenAIAssistantsAPIResponse
from .gateway_text_embedding_response import GatewayTextEmbeddingResponse



from typing import List
from azure.identity import DefaultAzureCredential
from langchain_core.retrievers import BaseRetriever
from foundationallm.config import Configuration
from foundationallm.services.gateway_text_embedding import GatewayTextEmbeddingService
from .azure_ai_search_service_retriever import AzureAISearchServiceRetriever
from foundationallm.models.agents import KnowledgeManagementIndexConfiguration

class RetrieverFactory:
    """
    Factory class for determine which retriever to use.
    """
    def __init__(
        self,
        index_configurations: List[KnowledgeManagementIndexConfiguration],
        gateway_text_embedding_service: GatewayTextEmbeddingService,
        config: Configuration):

        self.config = config
        self.index_configurations = index_configurations
        self.gateway_text_embedding_service = gateway_text_embedding_service
        
    def get_retriever(self) -> BaseRetriever:
        """
        Returns the retriever to use for completion requests.

        Returns
        -------
        BaseRetriever
            Returns the concrete initialization of a vectorstore retriever.
        """               
        credential = DefaultAzureCredential(exclude_environment_credential=True)                                   
  
        """
        # use indexing profile to build the retriever (current only supporting Azure AI Search)
        top_n = self.indexing_profile.settings.top_n
        filters = self.indexing_profile.settings.filters
        """

        retriever = AzureAISearchServiceRetriever(
            config=self.config,
            index_configurations=self.index_configurations,
            gateway_text_embedding_service=self.gateway_text_embedding_service
        )

        return retriever



import os
import logging
import json
from tenacity import retry, wait_random_exponential, stop_after_attempt, RetryError
from azure.appconfiguration.provider import (
    AzureAppConfigurationKeyVaultOptions,
    load
)
from azure.identity import DefaultAzureCredential

class Configuration():
    def __init__(self):
        """Init"""
        try:
            app_config_uri = os.environ['FOUNDATIONALLM_APP_CONFIGURATION_URI']
        except Exception as e:
            raise e

        credential = DefaultAzureCredential(
            exclude_environment_credential=True)

        # Connect to Azure App Configuration.
        self.__config = load(endpoint=app_config_uri, credential=credential,
                             key_vault_options=
                                AzureAppConfigurationKeyVaultOptions(credential=credential))

    def get_value(self, key: str) -> str:
        """
        Retrieves the value from Azure App Configuration.
        Otherwise, retrieves the value from the environment variable.
        If the value is not found the method raises an exception.

        Parameters
        ----------
        - key : str
            The key name of the configuration setting to retrieve.
        
        Returns
        -------
        The configuration value

        Raises an exception if the configuration value is not found.
        """
        if key is None:
            raise Exception('The key parameter is required for Configuration.get_value().')

        value = None

        # will have future usage with Azure App Configuration
        # if foundationallm-configuration-allow-environment-variables exists and is True, 
        #   then the environment variables will be checked first, then KV
        # if foundationallm-configuration-allow-environment-variables does not exist 
        #   OR foundationallm-configuration-allow-environment-variables is False, 
        #   then check App config and then KV
        allow_env_vars = False
        if "foundationallm-configuration-allow-environment-variables" in os.environ:
            allow_env_vars = bool(os.environ[
                    "foundationallm-configuration-allow-environment-variables"
                    ])

        if allow_env_vars is True:
            value = os.environ.get(key)

        if value is None:
            try:
                value = self.__get_config_with_retry(name=key)
            except Exception as e:
                pass

        if value is not None:
            return value
        else:
            raise Exception(f'The configuration variable {key} was not found.')

    def get_feature_flag(self, key: str) -> bool:
        """
        Retrieves the feature flag from Azure App Configuration.
        If the value is not found, returns false.
        Otherwise, retrieves the enabled value of the feature flag.

        Parameters
        ----------
        - key : str
            The key name of the feature flag to retrieve.
        
        Returns
        -------
        The enabled value of the feature flag
        
        """
        if key is None:
            raise KeyError('The key parameter is required for Configuration.get_feature_flag().')

        value = False

        if "FeatureManagementFeatureFlags" in self.__config.keys():
            if key in self.__config["FeatureManagementFeatureFlags"].keys():
                try:
                    feature_flag_setting = self.__config["FeatureManagementFeatureFlags"][key]
                    obj = json.loads(feature_flag_setting)
                    value = obj["enabled"]
                except Exception as e:
                    pass

        return value

    def __retry_before_sleep(self, retry_state):
        # Log the outcome of each retry attempt.
        message = f"""Retrying {retry_state.fn}:
                        attempt {retry_state.attempt_number}
                        ended with: {retry_state.outcome}"""
        if retry_state.outcome.failed:
            ex = retry_state.outcome.exception()
            message += f"; Exception: {ex.__class__.__name__}: {ex}"
        if retry_state.attempt_number < 1:
            logging.info(message)
        else:
            logging.warning(message)

    # Retry with jitter on transient errors. Initially up to 2^x * 1 seconds between each retry
    # until the range reaches 30 seconds, then randomly up to 60 seconds afterwards.
    # Stop after five retry attempts.
    @retry(
        wait=wait_random_exponential(multiplier=1, max=5),
        stop=stop_after_attempt(5),
        before_sleep=__retry_before_sleep
    )
    def __get_config_with_retry(self, name):
        try:
            return self.__config[name]
        except RetryError:
            pass
